----------------------------------------------------------------------------
-- Ten and Three
-- Copyright (c) 2016 Rikard Glans (natch@brainlag.se)
----------------------------------------------------------------------------
-- Time-stamp: <2016-12-08 02:37:02>
----------------------------------------------------------------------------
local ADDON_NAME = "Ten and Three"

-- Frame setup
local TAT = CreateFrame("Frame", "TenAndThreeFrame", UIParent)
TAT:SetScript("OnEvent", function(_, event, ...) TAT[event](TAT, ...) end)
TAT:RegisterEvent("ADDON_LOADED")
TAT:RegisterEvent("VARIABLES_LOADED")
TAT:RegisterEvent("PLAYER_ENTERING_WORLD")
TAT:RegisterEvent("PLAYER_EQUIPMENT_CHANGED")

TAT.cpanel = CreateFrame("Frame", "TenAndThreeConfigFrame", UIParent)
TAT.cpanel.name = ADDON_NAME
InterfaceOptions_AddCategory(TAT.cpanel)
----------------------------------------------------------------------------
-- Constants
local slots = {
  ["shoulder"] = 3,
  ["hands"]    = 10
}

local slotNames = {
  [slots.shoulder] = "shoulder",
  [slots.hands]    = "hands"
}
----------------------------------------------------------------------------
-- Variables
local ndebug   = false
local player   = nil
local warnings = {[slots.shoulder] = false, [slots.hands] = false}
----------------------------------------------------------------------------
-- Events
function TAT:ADDON_LOADED(msg)
  if(msg == "Natch_TenAndThree") then
    self:UnregisterEvent("ADDON_LOADED")

    _G.SLASH_NATCH_TENANDTHREE1          = "/tat"
    _G.SLASH_NATCH_TENANDTHREE2          = "/tenandthree"
    _G.SLASH_NATCH_TENANDTHREE3          = "/103"
    _G.SlashCmdList["NATCH_TENANDTHREE"] = function(...) self:SlashCMD(...) end

    self.ADDON_LOADED = nil
  end
end

function TAT:VARIABLES_LOADED()
  self:UnregisterEvent("VARIABLES_LOADED")

  player = UnitName("player").."@"..GetRealmName()

  if(not TenAndThree_Data) then
    TenAndThree_Data = {
      ["annoying"] = false,
      ["ultraannoying"] = false,
      ["msbt"] = true
    }
  end

  if(not TenAndThree_Data[player]) then
    TenAndThree_Data[player] = {
      ["enabled"] = true,
      ["shoulder"] = true,
      ["hands"] = true
    }
  end

  TAT:SetupConfig()

  self.VARIABLES_LOADED = nil
end

function TAT:PLAYER_ENTERING_WORLD()
  self:ScanForEnchants()
end

function TAT:PLAYER_EQUIPMENT_CHANGED()
  self:ScanForEnchants()
end
----------------------------------------------------------------------------
-- Configuration panel
local function CreateCheckbox(text, variable, usersetting, disabled, tooltiptext, parent)
  usersetting = usersetting or false
  disabled = disabled or false
  parent = parent or TAT.cpanel

  local template = "InterfaceOptionsCheckButtonTemplate"
  local checkbox = CreateFrame("CheckButton", "TATOption" .. variable, parent, template)

  checkbox.GetValue = function(self)
    local val

    if usersetting then
      val = TenAndThree_Data[player][variable]
    else
      val = TenAndThree_Data[variable]
    end

    return val
  end

  checkbox.SetValue = function(self, value)
    if usersetting then
      TenAndThree_Data[player][variable] = value
    else
      TenAndThree_Data[variable] = value
    end
  end

  checkbox:SetScript("OnClick", function(self) self:SetValue(self:GetChecked()) end)
  checkbox:SetChecked(checkbox:GetValue())
  checkbox.label = _G[checkbox:GetName() .. "Text"]
  checkbox.label:SetText(text)

  if disabled == nil then
    checkbox:Disable()
    checkbox.tooltipText = tooltiptext
  end

  if tooltiptext and disabled == 1 then
    checkbox.tooltipText = tooltiptext
  end

  return checkbox
end

local buttons = 1
local function CreateButton(text, action, parent)
  parent = parent or TAT.cpanel

  local button = CreateFrame("Button", "TATButton"..buttons, parent, "OptionsButtonTemplate")
  button:SetScript("OnClick", action)
  button.label = _G[button:GetName() .. "Text"]
  button.label:SetText(text)
  button:SetWidth(80)
  button:SetHeight(22)

  buttons = buttons + 1

  return button
end

function TAT:SetupConfig()
  TAT.cpanel.option1 = CreateCheckbox("Warnings enabled (character)", "enabled", true)
  TAT.cpanel.option2 = CreateCheckbox("Shoulder warning enabled (character)", "shoulder", true)
  TAT.cpanel.option3 = CreateCheckbox("Hands warning enabled (character)", "hands", true)

  TAT.cpanel.option4 = CreateCheckbox("Annoying Mode (global)", "annoying", false, 1, "Annoying mode prints the warning to every text area.")
  TAT.cpanel.option5 = CreateCheckbox("Ultra Annoying Mode (global)", "ultraannoying", false, 1, "Ultra annoying mode prints several warnings to every text area.")
  TAT.cpanel.option6 = CreateCheckbox("Use Mik's Scrolling Battle Text? (global)", "msbt", false, MikSBT or nil, "Mik's Scrolling Battle Text not found.")

  TAT.cpanel.option1:SetPoint("TOPLEFT", 16, -16)
  TAT.cpanel.option2:SetPoint("TOPLEFT", TAT.cpanel.option1, "BOTTOMLEFT")
  TAT.cpanel.option3:SetPoint("TOPLEFT", TAT.cpanel.option2, "BOTTOMLEFT")

  TAT.cpanel.option4:SetPoint("TOPLEFT", 16, -128)
  TAT.cpanel.option5:SetPoint("TOPLEFT", TAT.cpanel.option4, "BOTTOMLEFT")
  TAT.cpanel.option6:SetPoint("TOPLEFT", TAT.cpanel.option5, "BOTTOMLEFT")

  -- test button for annoying mode
  testAM = function()
    local save1 = TenAndThree_Data.annoying
    local save2 = TenAndThree_Data.ultraannoying
    TenAndThree_Data.annoying = true
    TenAndThree_Data.ultraannoying = false

    TAT:Warn("Testing Annoying Mode.")

    TenAndThree_Data.annoying = save1
    TenAndThree_Data.ultraannoying = save2
  end

  TAT.cpanel.buttonAM = CreateButton("Test", testAM)
  TAT.cpanel.buttonAM:SetPoint("LEFT", TAT.cpanel.option4, "RIGHT", TAT.cpanel.option4:GetWidth()*7, 0)

  -- test button for ultra annoying mode
  testUAM = function()
    local save = TenAndThree_Data.ultraannoying
    TenAndThree_Data.ultraannoying = true
    TAT:Warn("Testing Ultra Annoying Mode.")
    TenAndThree_Data.ultraannoying = save
  end

  TAT.cpanel.buttonUAM = CreateButton("Test", testUAM)
  TAT.cpanel.buttonUAM:SetPoint("LEFT", TAT.cpanel.option5, "RIGHT", TAT.cpanel.option5:GetWidth()*7, 0)
end

-- Functions
function TAT:ScanForEnchants()
  if TenAndThree_Data[player].enabled == false then return end

  for slot, item in pairs(slotNames) do
    local link = GetInventoryItemLink("player", slot)

    if link then
      local eid = string.match(link, "item:%d+:(%d+):")

      if not eid and warnings[slot] == false and TenAndThree_Data[player][item] == true then
        self:Warn(item .. " is missing enchant!")
        warnings[slot] = true
      end
    else
      self:print("Couldn't find item in slot: " .. string.lower(item) .. ".")
    end
  end
end

function TAT:ResetWarnings()
  warnings = {[slots.shoulder] = false, [slots.hands] = false}
end

-- Print warning with desired method.
function TAT:Warn(str)
  local msg = str

  if(MikSBT ~= nil and TenAndThree_Data.msbt == true) then
    if(TenAndThree_Data.annoying == true or TenAndThree_Data.ultraannoying == true) then
      for scrollAreaKey, _ in MikSBT.IterateScrollAreas() do
        if(TenAndThree_Data.ultraannoying == true) then
          MikSBT.DisplayMessage(msg, scrollAreaKey)
          MikSBT.DisplayMessage(msg, scrollAreaKey)
          MikSBT.DisplayMessage(msg, scrollAreaKey)
          MikSBT.DisplayMessage(msg, scrollAreaKey)
        end

        MikSBT.DisplayMessage(msg, scrollAreaKey)
      end

      if(TenAndThree_Data.ultraannoying == true) then
        self:print(msg)
        self:print(msg)
        self:print(msg)
        self:print(msg)
      end

      self:print(msg)
    else
      MikSBT.DisplayMessage(msg)

      self:print(msg)
    end

    return
  else
    if(TenAndThree_Data.ultraannoying == true) then
      UIErrorsFrame:AddMessage(msg, 1, 1, 0)
      UIErrorsFrame:AddMessage(msg, 1, 1, 0)
      UIErrorsFrame:AddMessage(msg, 1, 1, 0)
      UIErrorsFrame:AddMessage(msg, 1, 1, 0)

      self:print(msg)
      self:print(msg)
      self:print(msg)
      self:print(msg)
    end

    UIErrorsFrame:AddMessage(msg, 1, 1, 0)
    self:print(msg)
  end
end

-- Parse command line.
function TAT:SlashCMD(str)
  local cmd, arg = CmdArg(string.lower(str))

  if(not cmd or cmd == "") then
    InterfaceOptionsFrame_OpenToCategory(ADDON_NAME)
    InterfaceOptionsFrame_OpenToCategory(ADDON_NAME)
  elseif(cmd == "check" or cmd == "c") then
    self:ResetWarnings()
    self:ScanForEnchants()
  elseif(cmd == "reset" or cmd == "r") then
    self:ResetWarnings()
    self:print("Warnings reset.")
  elseif(cmd == "help" or cmd == "h") then
    self:print("Natch's Ten and Three")
    self:print("/tat   - options")
    self:print("/tat c - check enchants")
  else
    InterfaceOptionsFrame_OpenToCategory(ADDON_NAME)
    InterfaceOptionsFrame_OpenToCategory(ADDON_NAME)
  end
end

function TAT:print(str, ...)
  SELECTED_CHAT_FRAME:AddMessage("TenAndThree: " .. str, ...)
end

function TAT:debug(str, ...)
  if(ndebug == true) then
    SELECTED_CHAT_FRAME:AddMessage("TAT DEBUG: " .. str, ...)
  end
end

-- Function: CmdArg(string)
-- Usage: local cmd, arg = CmdArg(command-line-string)
function CmdArg(str)
  local argv = {}
  local s,e  = string.find(str, "%s+")

  if(not s or not e) then
    argv[1] = str
    argv[2] = nil
  else
    argv[1] = string.sub(str, 1, s-1)
    argv[2] = string.sub(str, e+1, -1)
  end

  return string.lower(argv[1]), argv[2]
end
